AltimateChat = {}

local _G = _G;

SLASH_ALTIMATECHAT1 = "/altchat"

local CMaxLevelNo = 80
local CColourDefaultMessage = "00FFFF00"

local ACLoginCountdown = 3
local ACDoRosterRefresh = false

local ACGuildChatHooks = {}
local ACOptionsList = {}
local ACUnknownMainsList = {}
local ACGuildMemberList = {}

local TmpMainToonTextColourAlpha, TmpMainToonTextColourRed, TmpMainToonTextColourGreen, TmpMainToonTextColourBlue

local AltimateChat_EventFrame = CreateFrame("FRAME")

------------------------------------------------------------------------------------
-- Main form functionality ---------------------------------------------------------
------------------------------------------------------------------------------------

local function AltChatAddMessage(self, message, ...)

	-- Check to see if this particular message is from guild chat, and bail immediately if it isnt.
	local IsGuildMessage = string.match(message, "|Hchannel:GUILD")
	if IsGuildMessage == nil then
		return ACGuildChatHooks[self](self, message, ...)
	end

	-- Determine some individual components of our final message so that we can reassemble it later.
	local GuildChannelLink = "|Hchannel:GUILD|h[Guild]|h"
	local PlayerAndRealmName = string.match(message, "|Hplayer:(.-):")
	local PlayerName = string.gsub(PlayerAndRealmName, "-(.*)", "")
	local _, ChatMessage = string.match(message, "|Hplayer:(.-):%s(.*)")
	local PlayerClass, MainToonName, LevelNo = AltimateChat:GetPlayerClassFromPlayerName( PlayerName )
	local MainPlayerClass = AltimateChat:GetPlayerClassFromPlayerName( MainToonName )
	local _, _, _, HexColour = GetClassColor( PlayerClass )
	local _, _, _, MainHexColour = GetClassColor( MainPlayerClass )
	local PlayerLink = "|Hplayer:" .. PlayerName .. "|h[" .. PlayerName .. "]|h"
	
	local LevelNoStr = ""
	if ACOptionsList["ShowPlayerLevel"] then
		if ( tonumber(LevelNo) > 0 ) and ( tonumber(LevelNo) ~= CMaxLevelNo ) then
			LevelNoStr = " (" .. LevelNo .. ")"
		else	
			LevelNoStr = ""
		end	
	else
		LevelNoStr = ""
	end
	
	-- If we have an associated main toon name, then add some colour to it.
	local Index = -1
	if MainToonName ~= "" then
		if MainPlayerClass == "" then
			-- Search our personal list instead
			Index = AltimateChat:FindPlayerInList(MainToonName, ACUnknownMainsList)
			
			if Index ~= 0 then
				_, _, _, MainHexColour = GetClassColor( ACUnknownMainsList[Index].ClassName )
			end
		end
		
		MainToonName = " (" .. MainToonName .. ")"
		if Index ~= 0 then
			if ACOptionsList["ShowMainToonsInTheirClassColour"] then
				MainToonName = "|c" .. MainHexColour .. MainToonName
			elseif ACOptionsList["ShowMainToonsInThisColour"] then
				MainHexColour = string.format("%02x", ACOptionsList["MainToonTextColourAlpha"] * 255) ..
				                string.format("%02x", ACOptionsList["MainToonTextColourRed"] * 255) ..
				                string.format("%02x", ACOptionsList["MainToonTextColourGreen"] * 255) ..
								string.format("%02x", ACOptionsList["MainToonTextColourBlue"] * 255) 
				MainToonName = "|c" .. MainHexColour .. MainToonName
			end
		end	
	end

	if ( GuildChannelLink ~= nil ) and ( HexColour ~= nil ) and ( PlayerLink ~= nil ) and ( MainToonName ~= nil ) and ( ChatMessage	~= nil ) then
		return ACGuildChatHooks[self](self, GuildChannelLink .. " " .. "|c" .. HexColour .. PlayerLink .. LevelNoStr .. "|r" .. MainToonName .. "|r: " .. ChatMessage, ...)
	else	
		return ACGuildChatHooks[self](self, message, ...)
	end
	
end

------------------------------------------------------------------------------------

function AltimateChat_OnSecond()

	if ACLoginCountdown > 0 then
		ACLoginCountdown = ACLoginCountdown - 1
		if ACLoginCountdown == 0 then
			ACGuildMemberList = AltimateChat:PopulateGuildList(ACGuildMemberList)
		end
	end

	if ACOptionsList["ShowWhenAPlayerHasJustLevelled"] then
		if ACLoginCountdown == 0 then
			if time() % 12 == 0 then
				if IsInGuild() then
					C_GuildInfo.GuildRoster()
				end
			end	
		end
	end

end

------------------------------------------------------------------------------------

local function AltimateChat_EventHandler(Self, Event, ...)

	if Event == "PLAYER_LOGIN" then
		AltimateChat_OptionsFrame:Hide()		
		AltimateChat_OptionsFrame:SetBackdrop( { bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background-Dark", 
												 edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
												 tile = true, tileSize = 16, edgeSize = 16, 
												 insets = { left = 4, right = 4, top = 4, bottom = 4 } } )
		AltimateChat_OptionsFrame:SetBackdropColor(0, 0, 0, 0.7)
		AltimateChat_OptionsFrame:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)		
	
		-- Load from our global variables
		ACOptionsList = AltimateChat_OptionsList	
		ACUnknownMainsList = AltimateChat_UnknownMainsList
		
		-- If running for the first time ever, initialise some lists
		if ACOptionsList == nil then
			ACOptionsList = {}
		end	
		if ACOptionsList["ShowQuestionMarkForUnknownPlayers"] == nil then
			ACOptionsList["ShowQuestionMarkForUnknownPlayers"] = true
		end	
		if ACOptionsList["ShowPlayerLevel"] == nil then
			ACOptionsList["ShowPlayerLevel"] = true
		end
		if ACOptionsList["ShowWhenAPlayerHasJustLevelled"] == nil then
			ACOptionsList["ShowWhenAPlayerHasJustLevelled"] = true
		end
		if ACOptionsList["ShowStartupMessage"] == nil then
			ACOptionsList["ShowStartupMessage"] = true
		end
		if ACOptionsList["ShowMainToonsInTheirClassColour"] == nil then
			ACOptionsList["ShowMainToonsInTheirClassColour"] = true
		end
		if ACOptionsList["ShowMainToonsInThisColour"] == nil then
			ACOptionsList["ShowMainToonsInThisColour"] = false
		end
		if ACOptionsList["MainToonTextColourAlpha"] == nil then
			ACOptionsList["MainToonTextColourAlpha"] = 1
		end
		if ACOptionsList["MainToonTextColourRed"] == nil then
			ACOptionsList["MainToonTextColourRed"] = 1
		end
		if ACOptionsList["MainToonTextColourGreen"] == nil then
			ACOptionsList["MainToonTextColourGreen"] = 0
		end
		if ACOptionsList["MainToonTextColourBlue"] == nil then
			ACOptionsList["MainToonTextColourBlue"] = 0
		end
		
		if ACUnknownMainsList == nil then
			ACUnknownMainsList = {}
		end
		
		if ACOptionsList["ShowStartupMessage"] then
			DEFAULT_CHAT_FRAME:AddMessage("|c" .. CColourDefaultMessage .. "Altimate Chat (Classic) - Loaded (type /altchat for options)|r")
		end	
	elseif Event == "PLAYER_LOGOUT" then
		-- Save back to our global variables
		AltimateChat_OptionsList = ACOptionsList
		AltimateChat_UnknownMainsList = ACUnknownMainsList
	elseif Event == "GUILD_ROSTER_UPDATE" then
		AltimateChat:CheckIfSomeoneLevelled()
	end

end		

------------------------------------------------------------------------------------

function AltimateChat:PopulateGuildList(AList)

	local Index, PlayerName, PlayerAndRealmName, LevelNo
	local WoWGuildMemberCount = GetNumGuildMembers()
	
	AList = {}

	-- Loop through our current WoW guild roster, and add all toons.
	for i = 1, WoWGuildMemberCount do
		PlayerAndRealmName, _, _, LevelNo = GetGuildRosterInfo(i)
		PlayerName = string.gsub(PlayerAndRealmName, "-(.*)", "")
	
		Index = #AList + 1
		AList[Index] = {}
		AList[Index].PlayerName = PlayerName
		AList[Index].LevelNo = LevelNo
	end
	
	return AList
	
end

------------------------------------------------------------------------------------

function AltimateChat:CheckIfSomeoneLevelled()

	if ACOptionsList["ShowWhenAPlayerHasJustLevelled"] then	
		local NewGuildMemberList = {}
		local Index
		
		NewGuildMemberList = AltimateChat:PopulateGuildList(NewGuildMemberList)

		for i = 1, #NewGuildMemberList do

			Index = AltimateChat:FindPlayerInList(NewGuildMemberList[i].PlayerName, ACGuildMemberList)
			if Index == 0 then
				-- Add the player to the main list, if they've just logged in
				Index = #ACGuildMemberList + 1
				ACGuildMemberList[Index] = {}
				ACGuildMemberList[Index].PlayerName = NewGuildMemberList[i].PlayerName
				ACGuildMemberList[Index].LevelNo = NewGuildMemberList[i].LevelNo		
			else
				if NewGuildMemberList[i].LevelNo ~= ACGuildMemberList[Index].LevelNo then
					ACGuildMemberList[Index].LevelNo = NewGuildMemberList[i].LevelNo
					
					local _, MainToonName = AltimateChat:GetPlayerClassFromPlayerName( ACGuildMemberList[Index].PlayerName )
					if MainToonName ~= "" then
						MainToonName = " (" .. MainToonName .. ")"
					end
					DEFAULT_CHAT_FRAME:AddMessage("|c" .. CColourDefaultMessage .. "***** " .. ACGuildMemberList[Index].PlayerName .. MainToonName .. " has reached level " .. ACGuildMemberList[Index].LevelNo .. "! *****")
				end
			end
			
		end
	end	

end

------------------------------------------------------------------------------------

function AltimateChat:FindPlayerInList(ASourcePlayerName, AList)

	-- Find the player, if they already exist in our list

	local PlayerName = AltimateChat:ProperCase(ASourcePlayerName)

	for i = 1,#AList do
		if PlayerName == AList[i].PlayerName then
			return i
		end
	end
	
	return 0

end

------------------------------------------------------------------------------------

function AltimateChat:GetPlayerClassFromPlayerName(APlayerName)

	local WoWGuildMemberCount = GetNumGuildMembers()
	local MainToonName

	for i = 1, WoWGuildMemberCount do
		local PlayerNameAndRealm, _, _, LevelNo, _, _, Note, _, _, _, PlayerClass = GetGuildRosterInfo(i)
		local PlayerName = string.gsub(PlayerNameAndRealm, "-(.*)", "")	

		if PlayerName == APlayerName then
			MainToonName = AltimateChat:GetMainToonNameFromNote(Note)
			return PlayerClass, MainToonName, LevelNo
		end
	end	

	return "", "", 0

end

------------------------------------------------------------------------------------

function AltimateChat:GetMainToonNameFromNote(ANote)

	local Note, Word1, Word2 = ANote, "", ""

	Note = string.gsub(Note, "'s", "")
	Note = string.gsub(Note, "'", "")

	for Word in string.gmatch(Note, "%w+") do
		if Word1 == "" then
			Word1 = Word
		elseif Word2 == "" then
			Word2 = Word
		end
	end

	if ( string.lower(Word1) == 'main' ) then
		return ""
	elseif ( string.lower(Word2) == 'alt' ) or ( string.lower(Word2) == 'main' ) then
		return Word1
	elseif ACOptionsList["ShowQuestionMarkForUnknownPlayers"] then
		return "?"
	else	
		return ""
	end

end

------------------------------------------------------------------------------------

function AltimateChat:AddPlayerToUnknownMainsList(APlayerName, AClassName)

	-- Add a player to the unknown mains list, or modify one that's existing there

	local Index = AltimateChat:FindPlayerInList(APlayerName, ACUnknownMainsList)

	if Index == 0 then
		Index = #ACUnknownMainsList + 1
		ACUnknownMainsList[Index] = {}
	end
	
	ACUnknownMainsList[Index].PlayerName = AltimateChat:ProperCase(APlayerName)
	ACUnknownMainsList[Index].ClassName = string.upper(AClassName)
	
	DEFAULT_CHAT_FRAME:AddMessage("|c" .. CColourDefaultMessage .. ACUnknownMainsList[Index].PlayerName .. " added as a " .. AClassName .. "|r")

end

------------------------------------------------------------------------------------

function AltimateChat:IsAValidClass(AClassName)

	local ClassName = string.lower(AClassName)

	return ( ClassName == "druid" ) or ( ClassName == "hunter" ) or ( ClassName == "mage" ) or 
	       ( ClassName == "paladin" ) or ( ClassName == "priest" ) or ( ClassName == "rogue" ) or 
	       ( ClassName == "shaman" ) or ( ClassName == "warlock" ) or ( ClassName == "warrior" ) or 
	       ( ClassName == "deathknight" )

end

------------------------------------------------------------------------------------

function AltimateChat:ProperCase(AStr)

	local First = string.sub(AStr, 1, 1)
	local Rest = string.sub(AStr, 2)
	
	return string.upper(First) .. string.lower(Rest)

end

------------------------------------------------------------------------------------
-- Options form functionality ------------------------------------------------------
------------------------------------------------------------------------------------

function AltimateChat:OnOpenOptionsButtonClick()

	-- Set all UI options prior to opening the options form
	
	AltimateChat_ShowQuestionMarkForUnknownPlayersCheckboxText:SetText("Show Question Mark For Unknown Players")
	AltimateChat_ShowPlayerLevelCheckboxText:SetText("Show Player Level")
	AltimateChat_ShowWhenAPlayerHasJustLevelledCheckboxText:SetText("Show When A Player Has Just Levelled")
	AltimateChat_ShowStartupMessageCheckboxText:SetText("Show Startup Message")
	AltimateChat_ShowMainToonsInTheirClassColourCheckboxText:SetText("Show (Main) Toons In Their Class Colour")
	AltimateChat_ShowMainToonsInThisColourCheckboxText:SetText("Show (Main) Toons In This Colour")
	
	AltimateChat_ShowQuestionMarkForUnknownPlayersCheckboxText:SetTextColor(1,1,1)
	AltimateChat_ShowPlayerLevelCheckboxText:SetTextColor(1,1,1)
	AltimateChat_ShowWhenAPlayerHasJustLevelledCheckboxText:SetTextColor(1,1,1)
	AltimateChat_ShowStartupMessageCheckboxText:SetTextColor(1,1,1)
	AltimateChat_ShowMainToonsInTheirClassColourCheckboxText:SetTextColor(1,1,1)
	AltimateChat_ShowMainToonsInThisColourCheckboxText:SetTextColor(1,1,1)
	
	AltimateChat_ShowQuestionMarkForUnknownPlayersCheckbox:SetChecked(ACOptionsList["ShowQuestionMarkForUnknownPlayers"])
	AltimateChat_ShowPlayerLevelCheckbox:SetChecked(ACOptionsList["ShowPlayerLevel"])
	AltimateChat_ShowWhenAPlayerHasJustLevelledCheckbox:SetChecked(ACOptionsList["ShowWhenAPlayerHasJustLevelled"])
	AltimateChat_ShowStartupMessageCheckbox:SetChecked(ACOptionsList["ShowStartupMessage"])
	AltimateChat_ShowMainToonsInTheirClassColourCheckbox:SetChecked(ACOptionsList["ShowMainToonsInTheirClassColour"])
	AltimateChat_ShowMainToonsInThisColourCheckbox:SetChecked(ACOptionsList["ShowMainToonsInThisColour"])

	AltimateChat_OptionsFrame:Show()

end

------------------------------------------------------------------------------------

function AltimateChat:OnCloseOptionsButtonClick()

	-- Set the options in the table after closing the options form
	
	ACOptionsList["ShowQuestionMarkForUnknownPlayers"] = AltimateChat_ShowQuestionMarkForUnknownPlayersCheckbox:GetChecked()
	ACOptionsList["ShowPlayerLevel"] = AltimateChat_ShowPlayerLevelCheckbox:GetChecked()
	ACOptionsList["ShowWhenAPlayerHasJustLevelled"] = AltimateChat_ShowWhenAPlayerHasJustLevelledCheckbox:GetChecked()
	ACOptionsList["ShowStartupMessage"] = AltimateChat_ShowStartupMessageCheckbox:GetChecked()
	ACOptionsList["ShowMainToonsInTheirClassColour"] = AltimateChat_ShowMainToonsInTheirClassColourCheckbox:GetChecked()
	ACOptionsList["ShowMainToonsInThisColour"] = AltimateChat_ShowMainToonsInThisColourCheckbox:GetChecked()
	
	AltimateChat_OptionsFrame:Hide()

end

------------------------------------------------------------------------------------

function ColorPickerCallbackOk(restore)

	ACOptionsList["MainToonTextColourAlpha"] = OpacitySliderFrame:GetValue()
	ACOptionsList["MainToonTextColourRed"], ACOptionsList["MainToonTextColourGreen"], ACOptionsList["MainToonTextColourBlue"] = ColorPickerFrame:GetColorRGB()

end

------------------------------------------------------------------------------------

function ColorPickerCallbackCancel(restore)

	ACOptionsList["MainToonTextColourAlpha"] = TmpMainToonTextColourAlpha
	ACOptionsList["MainToonTextColourRed"] = TmpMainToonTextColourRed
	ACOptionsList["MainToonTextColourGreen"] = TmpMainToonTextColourGreen
	ACOptionsList["MainToonTextColourBlue"] = TmpMainToonTextColourBlue

end

------------------------------------------------------------------------------------

function AltimateChat:OnColourPickerButtonClick()

	TmpMainToonTextColourAlpha = ACOptionsList["MainToonTextColourAlpha"]
	TmpMainToonTextColourRed = ACOptionsList["MainToonTextColourRed"]
	TmpMainToonTextColourGreen = ACOptionsList["MainToonTextColourGreen"]
	TmpMainToonTextColourBlue = ACOptionsList["MainToonTextColourBlue"]

	ColorPickerFrame:SetColorRGB(TmpMainToonTextColourRed, TmpMainToonTextColourGreen, TmpMainToonTextColourBlue)
	ColorPickerFrame.Opacity = TmpMainToonTextColourAlpha
	ColorPickerFrame.func = ColorPickerCallbackOk
	ColorPickerFrame.opacityFunc = ColorPickerCallbackOk
	ColorPickerFrame.cancelFunc	= ColorPickerCallbackCancel

	ColorPickerFrame:Hide()
	ColorPickerFrame:Show()

end

------------------------------------------------------------------------------------

function SlashCmdList.ALTIMATECHAT(AMsg, AEditBox)

	local Word1, Word2 = "", "", ""
	local ParamCount = 0
	local PlayerName, ClassName = "", ""
	
	AMsg = string.lower(AMsg)

	for Word in string.gmatch(AMsg, "%S+") do
		ParamCount = ParamCount + 1
		
		if Word1 == "" then
			Word1 = Word
		elseif Word2 == "" then
			Word2 = Word
		end
	end

	if ( ParamCount == 0 ) or ( Word1 == "show" ) then
		AltimateChat:OnOpenOptionsButtonClick()
	elseif ParamCount == 2 then
		if AltimateChat:IsAValidClass(Word1) then
			AltimateChat:AddPlayerToUnknownMainsList(Word2, Word1)
		elseif AltimateChat:IsAValidClass(Word2) then
			AltimateChat:AddPlayerToUnknownMainsList(Word1, Word2)
		end
	end

end	

------------------------------------------------------------------------------------

for Index = 1, NUM_CHAT_WINDOWS do
    if Index ~= 2 then
        local ACGuildChatFrame = _G['ChatFrame'..Index]
        ACGuildChatHooks[ACGuildChatFrame] = ACGuildChatFrame.AddMessage  -- inherited call to cascade to
        ACGuildChatFrame.AddMessage = AltChatAddMessage
    end
end

AltimateChat_EventFrame:RegisterEvent("PLAYER_LOGIN")
AltimateChat_EventFrame:RegisterEvent("PLAYER_LOGOUT")
AltimateChat_EventFrame:RegisterEvent("GUILD_ROSTER_UPDATE")
AltimateChat_EventFrame:SetScript("OnEvent", AltimateChat_EventHandler)
C_Timer.NewTicker(1, AltimateChat_OnSecond)
